package space.eptaproject.vmodextras.block;

import edn.stratodonut.drivebywire.wire.ShipWireNetworkManager;
import edn.stratodonut.trackwork.tracks.forces.PhysicsTrackController;
import edn.stratodonut.trackwork.tracks.forces.SimpleWheelController;
import net.minecraft.core.BlockPos;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.level.block.Rotation;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.state.BlockState;

import org.joml.Vector3d;
import org.joml.Vector3dc;
import org.valkyrienskies.core.api.ships.ServerShip;
import org.valkyrienskies.mod.common.VSGameUtilsKt;
import space.eptaproject.vmodextras.trackwork.IPhysicsTrackController;
import space.eptaproject.vmodextras.trackwork.ISimpleWheelController;
import space.eptaproject.vmodextras.VmodExtras;


public class CompactBackupBlockEntity extends BlockEntity {
    private CompoundTag pendingBackupData;

    public CompactBackupBlockEntity(BlockPos p_155229_, BlockState p_155230_) {
        super(VmodExtras.COMPACT_BACKUP_BLOCK_ENTITY.get(), p_155229_, p_155230_);
    }

    @Override
    protected void saveAdditional(CompoundTag tag) {
        super.saveAdditional(tag);
        ServerShip ship = (ServerShip) VSGameUtilsKt.getShipObjectManagingPos(this.level, getBlockPos());
        if (ship != null) {
            {
                PhysicsTrackController physicsTrackController = ship.getAttachment(PhysicsTrackController.class);
                IPhysicsTrackController controller = (IPhysicsTrackController) physicsTrackController;
                if (physicsTrackController != null) {
                    CompoundTag PhysicsTrackControllerTag = new CompoundTag();

                    PhysicsTrackControllerTag.putFloat("SuspensionStiffness", controller.getSuspensionStiffness());
                    {
                        CompoundTag suspensionAdjust = new CompoundTag();

                        Vector3dc vec = controller.getSuspensionAdjust();

                        suspensionAdjust.putDouble("x", vec.x());
                        suspensionAdjust.putDouble("y", vec.y());
                        suspensionAdjust.putDouble("z", vec.z());

                        PhysicsTrackControllerTag.put("suspensionAdjust", suspensionAdjust);
                    }

                    tag.put("PhysicsTrackController", PhysicsTrackControllerTag);
                }
            }
            {
                SimpleWheelController simpleWheelController = ship.getAttachment(SimpleWheelController.class);
                ISimpleWheelController controller = (ISimpleWheelController) simpleWheelController;
                if (simpleWheelController != null) {
                    CompoundTag SimpleWheelControllerTag = new CompoundTag();

                    SimpleWheelControllerTag.putFloat("SuspensionStiffness", controller.getSuspensionStiffness());
                    {
                        CompoundTag suspensionAdjust = new CompoundTag();

                        Vector3dc vec = controller.getSuspensionAdjust();

                        suspensionAdjust.putDouble("x", vec.x());
                        suspensionAdjust.putDouble("y", vec.y());
                        suspensionAdjust.putDouble("z", vec.z());

                        SimpleWheelControllerTag.put("suspensionAdjust", suspensionAdjust);
                    }

                    tag.put("SimpleWheelController", SimpleWheelControllerTag);
                }
            }
        }

        ship = (ServerShip) VSGameUtilsKt.getShipManagingPos(this.level, this.getBlockPos());

        if (this.level != null) {
            if (this.pendingBackupData == null) {
                this.pendingBackupData = new CompoundTag();
            }

            ShipWireNetworkManager.get(ship).ifPresent((m) -> this.pendingBackupData.merge(m.serialiseToNbt(this.level, this.getBlockPos())));

            if (this.pendingBackupData != null) {
                tag.put("WireNetwork", this.pendingBackupData);
            }

        }
    }

    @Override
    public void load(CompoundTag tag) {
        super.load(tag);

        ServerShip ship = (ServerShip) VSGameUtilsKt.getShipObjectManagingPos(this.level, getBlockPos());
        if (ship != null) {
            if (tag.contains("SimpleWheelController")) {
                CompoundTag L = tag.getCompound("SimpleWheelController");
                SimpleWheelController controller2 = SimpleWheelController.getOrCreate(ship);
                if (L.contains("SuspensionStiffness")) {
                    ((ISimpleWheelController) controller2).setSuspensionStiffness(L.getFloat("SuspensionStiffness"));
                }
                if (L.contains("suspensionAdjust")) {
                    CompoundTag suspensionAdjust = L.getCompound("suspensionAdjust");

                    ((ISimpleWheelController) controller2).setSuspensionAdjust(new Vector3d(new Vector3d(suspensionAdjust.getDouble("x"), suspensionAdjust.getDouble("y"), suspensionAdjust.getDouble("z"))));
                }
            }

            if (tag.contains("PhysicsTrackController")) {
                CompoundTag L = tag.getCompound("PhysicsTrackController");
                PhysicsTrackController controller = PhysicsTrackController.getOrCreate(ship);
                if (L.contains("SuspensionStiffness")) {
                    ((IPhysicsTrackController) controller).setSuspensionStiffness(L.getFloat("SuspensionStiffness"));
                }
                if (L.contains("suspensionAdjust")) {
                    CompoundTag suspensionAdjust = L.getCompound("suspensionAdjust");


                    ((IPhysicsTrackController) controller).setSuspensionAdjust(new Vector3d(suspensionAdjust.getDouble("x"), suspensionAdjust.getDouble("y"), suspensionAdjust.getDouble("z")));
                }
            }
        }

        ship = (ServerShip) VSGameUtilsKt.getShipManagingPos(this.level, getBlockPos());
        if (ship != null) {
            if (tag.contains("WireNetwork", 10)) {
                ShipWireNetworkManager.loadIfNotExists(ship, level, tag.getCompound("WireNetwork"), this.getBlockPos(), Rotation.NONE);

                this.pendingBackupData = tag.getCompound("WireNetwork");
            }
        }
    }
}
